@@ -6,8 +6,9 @@ Matlab version:
     date:   26 Apr 2009
     course: Multivariable Control Systems
 """
-import numpy as np
+import copy
 
+import numpy as np
 ##############################################################################
 def blkdiag(x):
     if isinstance(x, (int, float, str, np.int64)): y = np.array([x])
@@ -31,6 +32,9 @@ def c_(x):
 def cmp(x, y, skipna=False, skipinf=False):
     if isinstance(x, list): x = np.array(x)
     if isinstance(y, list): y = np.array(y)
+    if x.dtype == 'bool': x = x.astype(int)
+    if y.dtype == 'bool': y = y.astype(int)
+    if isinstance(y, list): y = np.array(y)
     if skipna:
         ix = np.isnan(x) == False
         iy = np.isnan(y) == False
@@ -76,8 +80,9 @@ def delrc(x, i, j=None):
         res[i:, 0] = x[i+1:, 0]
     return res
 ##############################################################################
-def diff(x):
-    if x.shape[1] > 1: dx = x[1:, :] - x[:-1, :]
+def d_(x):
+    if len(x) < 2: return 0
+    if x.ndim > 1 and x.shape[1] > 1: dx = x[1:, :] - x[:-1, :]
     else:              dx = x[1:] - x[:-1]
     return dx
 ##############################################################################
@@ -161,7 +166,7 @@ def m_(x=[], size=None, size2=None, tp=None):
     # m_([])                --> array([], dtype=float64)
     
     try:
-        if size is None:    res = np.array(x)
+        if size is None: res = np.array(x)
         elif size2 is None:
             if isinstance(size, (int, np.int64, float)): size = [size, size]
             if len(size) == 1:  size = (int(size[0]),)
@@ -180,8 +185,11 @@ def m_(x=[], size=None, size2=None, tp=None):
     else: return res
 ###################################################################################
 def max1(x, axis=0, naskip=False):
+    if isinstance(x, list): x = m_(x)
     # Returns maximum element of an array and corresponding index
-    if naskip: x[np.isnan(x)] = min(x[np.invert(np.isnan(x))]) - 1
+    if naskip:
+        x = new(x)
+        x[np.isnan(x)] = min(x[np.invert(np.isnan(x))]) - 1
     if x.ndim == 2:
         N, m = x.shape
         if axis == 0:
@@ -195,7 +203,11 @@ def max1(x, axis=0, naskip=False):
             i1, i2 = np.where(x == xmax)
             ind = nans((N, 1))
             for i in range(N): ind[i, 0] = i2[i1 == i][0]
-        if any(np.array([N, m]) == 1):
+        if axis == -1: # max(max(x))
+            xmax = np.amax(np.amax(x))
+            i1, i2 = np.where(x == xmax)
+            ind = m_([i1, i2])
+        if any(m_([N, m]) == 1):
             xmax = xmax.flatten()[0]
             ind = ind.flatten()[0]
     else:
@@ -245,7 +257,14 @@ def nans(size, size2=None):
             raise Exception('In NANS(): Incorrect input parameters. Should be nans((<rows>, <cols>)), nans(<rows>, <cols>) or nans(<rows_cols>).')
     res = np.full(size, np.nan)
     return res
-###################################################################################
+##############################################################################
+def new(x):
+    x1 = copy.deepcopy(x)
+    del x
+    x = x1
+    del x1
+    return x
+##############################################################################
 def num2lst(a):
     # Converts array to list. If array is 2D, then i-th element of the list is a list consisting the elements of the i-th row of.
     if type(a) is np.ndarray: return [num2lst(x) for x in a]
