@@ -7,8 +7,7 @@ Matlab version:
     course: Multivariable Control Systems
 """
 import numpy as np
-from aislab.md_reg import*# needed for vif()
-from aislab.dp_feng.normlz import *
+from aislab.md_reg import *# needed for vif()
 from aislab.gnrl.bf import *
 
 
@@ -170,7 +169,7 @@ def betai(a, b, x):
         bi[ii] = 1 - bi[ii]*betacf(b[ii], a[ii], 1 - x[ii])/b[ii]
     return bi
 ##############################################################################
-def chi2(g, b):
+def chi2(g, b): # for binary dependent
     if g.ndim == 1: g = g.reshape(len(g), 1); b = b.reshape(len(b), 1)
     G = g.sum(axis=0)
     B = b.sum(axis=0)
@@ -199,6 +198,7 @@ def corf(x, y=None):
     return R
 ###################################################################################
 def covf(x, y=None):
+    from aislab.dp_feng.normlz import stdn
     x, __ = stdn(x)
     if y is None: y = x
     C = corf(x, y)
@@ -214,20 +214,22 @@ def dind(g, b):
 ##############################################################################
 def frat(y, y2, nw):
     if y.ndim == 1 and len(y) < 1: f = np.nan; return f
-    if y.ndim == 1:
-        m, n = len(y), 1
-    else:
-        m, n = y.shape
+    if y.ndim == 1: m = len(y)
+    else:           m = y.shape[0]
+    # y = y.astype('float')
     eps = 1e-6  # np.finfo(float).eps
     nw[nw < eps] = eps
-    ssy = sum(y2)
-    rc = sum(y.astype('float')**2/nw)
-    ys = sum(y)
     Nw = sum(nw)
-    s1 = rc - float(ys)**2/Nw
-    s2 = ssy - rc
-    f = s1/s2*(Nw - m)/(m - 1)
-    if type(f) != np.array: f = np.array([f])
+    ssy = sum(y2)
+    ys = m_(sum(y))
+    s3 = sum(y**2/nw)
+    s1 = s3 - ys**2/Nw
+    s2 = ssy - s3
+
+    import warnings
+    warnings.filterwarnings("ignore", category=RuntimeWarning)
+    
+    f = m_(s1/s2*np.max(np.hstack((0, Nw - m)))/(m - 1))
     return f
 ###################################################################################
 def gamln(a):
