@@ -39,16 +39,16 @@ def cmpr(x=None, y=None, d=False):
         elif isinstance(x, np.ndarray):
             pass
         else:
-            raise Exception('x must be scalar, np.ndarray or a list of scalars or np.ndarrays.')
+            raise Exception('In CMPR(): x must be scalar, np.ndarray or a list of scalars or np.ndarrays.')
         if isinstance(x, np.ndarray):
             y = np.load('cmpr_file.npy')
-            mad(x, y, dsp=True)
+            md(x, y, dsp=True)
         else:
             container = np.load('cmpr_file.npz')
             y = [container[key] for key in container]
             if isinstance(x, list):
                 for i in range(len(x)):
-                    mad(x[i], y[i], dsp=True)
+                    md(x[i], y[i], dsp=True)
     elif x is None and y is None and d:
         try:
             os.remove('cmpr_file.npy')
@@ -153,6 +153,7 @@ def md(x, y, dsp=True, prc=False):  # max difference - default is absolute, and
 
 ##############################################################################
 def nsinv(X, s_min=1e-12, met='EVD', mtype='PDMR'):
+    if np.any(np.isnan(X)): return nans(X.shape)
     if met=='SVD':
         U, S, V = np.linalg.svd(X, full_matrices=True)
         # if np.max(S) == 0:
@@ -198,7 +199,9 @@ def randn(size=1, m=0, s=1, seed=None):
     r = np.random.normal(loc=m, scale=s, size=size)
     return r
 ##############################################################################    
-def rng(x): return max(x) - min(x)
+def rng(x, d=False): 
+    if d:   return max(x) - min(x)
+    else:   return min(x), max(x)
 ##############################################################################    
 def sort(x, order='asscend'):
     sz = x.shape
