@@ -21,6 +21,19 @@ pd.set_option('precision', 5)
 # def gswlinr():
 ###################################################################################
 ###################################################################################
+def add_intrcpt(F, pm0, ivi, cnames, mdl_init):
+    flg_addint = 0
+    if mdl_init == 'empty' and isinstance(pm0, (int, float)) and pm0 == 0:
+        pm0 = 1
+        F = np.hstack((np.ones((F.shape[0], 1)), F))
+        ivi = np.hstack((0, 1 + ivi))
+        cnames = np.vstack(('intercept', cnames))
+        flg_addint = 1
+        wn.warn('In SWLINR(): Intercept is introduced in the model as the feature selection starts with "empty" initial model (the parameter mdl_init = \'empty\') that is equivalent to "intercept-only model".')
+    return F, pm0, ivi, cnames, flg_addint
+###################################################################################
+
+
 def bElimin(model, st0, met, cnames, s_min, SLS, crit_nbm, iterate):
     Nw, r, z, FY, FF, wF, ivi, ivo, n1, n2 = parin(st0, model[-1])
     # ----- Calculate significance of factors in the model -----
@@ -29,6 +42,7 @@ def bElimin(model, st0, met, cnames, s_min, SLS, crit_nbm, iterate):
         return model, iterate
     ivi0 = ivi
     ivo0 = ivo
+
     if ivi.shape[0] > 1:
         n1 = ivi.shape[0]
         n2 = n2 - 1
@@ -52,22 +66,20 @@ def bElimin(model, st0, met, cnames, s_min, SLS, crit_nbm, iterate):
         if p_Fpi > SLS:
             ivi = delrc(ivi0, ind)
             ivo = np.hstack((ivo, ivi0[ind]))
-            z = htmdl('BE1', model[-1], 'BE', n2, ivi, ivo, cnames, ivi0[ind], pmi, sti)
-            model = model + [z]
+            zz = htmdl('BE1', model[-1], 'BE', n2, ivi, ivo, cnames, ivi0[ind], pmi, sti)
+            model = model + [zz]
         else:
-            if met == 'BR':
-                iterate = 0
-            else:
-                model[-1]['FB'] = 'BE_stop'
+            if met == 'BR': iterate = 0
+            elif model[-1]['FB'] == 'BE': model[-1]['FB'] = 'BE_'
     elif met == 'BR':
         iterate = 0
     else:
-        model[-1]['FB'] = 'BE_stop'
+        model[-1]['FB'] = 'BE_'
     return model, iterate
 
 
 ###################################################################################
-def firstmdl(st0, mdl_init, s_min, cnames, ivi, rtp='PM'):
+def firstmdl(st0, mdl_init, s_min, cnames, ivi):
     if mdl_init == 'full':
         z = st0['z']
         FF = st0['FF']
@@ -77,102 +89,41 @@ def firstmdl(st0, mdl_init, s_min, cnames, ivi, rtp='PM'):
         ivo = np.array([]).astype(int)
         n2 = z
         pm, P = mdl(FF, FY, s_min)
-        st_ext = stmdl('BE1', [], FY, P, wF, n2, [], st0)
+        model0 = {'ivi': ivi,
+                  'ivo': ivo
+                 }
+        st_ext = stmdl(FB='BE1', model=model0, rtp='PM', FY=FY, P=P, wF=wF, n2=n2, n1=[], st0=st0)
         # st_ext2 = stmdl('FS1', [], FY, P, wF, n2, [], st0)
         model = [htmdl('BE1', [], 'BE', n2, ivi, ivo, cnames, [], pm, st_ext['ovr'])]
     elif mdl_init == 'empty':
         z = st0['z']
-        if rtp == 'PM':
-            cn = cnames[0]
-            FF = np.array([[st0['FF'][0, 0]]])
-            FY = np.array([st0['FY'][0]])
-            wF = np.array([st0['wF'][0, 0]])
-            ivi = m_([0])
-            ivo = np.arange(1, z)
-            n2 = 1
-        else:
-            cz = np.hstack((0, np.cumsum(z[:-1]))).astype(int)
-            cn = cnames[cz]
-            r = st0['r']
-            FF = zeros(r, r); FY = nans(r, 1); wF = nans(1, r); ivi = [None]*r; ivo = [None]*r
-            for i in np.arange(r):
-                ii = cz[i]
-                FF[i, i] = np.array([[st0['FF'][ii, ii]]])
-                FY[i] = np.array([st0['FY'][ii]])
-                wF[0, i] = np.array([st0['wF'][0, ii]])
-                ivi[i] = m_([ii])
-                ivo[i] = np.arange(1, z[i])
-            n2 = ones((r,))
+        cn = cnames[0]
+        FF = np.array([[st0['FF'][0, 0]]])
+        FY = np.array([st0['FY'][0]])
+        wF = np.array([st0['wF'][0, 0]])
+        ivi = m_([0])
+        ivo = np.arange(1, z)
+        n2 = 1
         pm, P = mdl(FF, FY, s_min)
         model0 = {'ivi': ivi,
                   'ivo': ivo
                  }
-        st_ext = stmdl(FB='FS1', model=model0, rtp=rtp, FY=FY, P=P, wF=wF, n2=n2, n1=[], st0=st0)
+        st_ext = stmdl(FB='FS1', model=model0, rtp='PM', FY=FY, P=P, wF=wF, n2=n2, n1=[], st0=st0)
         model = [htmdl('FS1', [], 'FS', n2, ivi, ivo, cnames, [], pm, st_ext['ovr'])]
     elif mdl_init == 'init':
         z = st0['z']
-        # ivo = np.array(list(set(np.arange(z)) - set(ivi.flatten())))
-        # n2 = ivi.shape[0]
-        # FF = sm_(st0['FF'], ivi, ivi)
-        # FY = sm_(st0['FY'], ivi)
-        # wF = sm_(st0['wF'], ivi)
-        # pm, P = mdl(FF, FY, s_min)
-        # model0 = {'ivi': ivi,
-        #           'ivo': ivo
-        #          }
-        # st_ext = stmdl(FB='FS1', model=model0, rtp=rtp, FY=FY, P=P, wF=wF, n2=n2, n1=[], st0=st0)
-        # model = [htmdl('FS1', [], 'FS', n2, ivi, ivo, cnames, [], pm, st_ext['ovr'])]
-        
-        if rtp == 'PM':
-            ivo = np.array(list(set(np.arange(z)) - set(ivi.flatten())))
-            n2 = ivi.shape[0]
-            cn = cnames[ivi]
-            FF = sm_(st0['FF'], ivi, ivi)
-            FY = sm_(st0['FY'], ivi)
-            wF = sm_(st0['wF'], ivi)
-        else:
-            r = st0['r']
-            if ivi is None:
-                cz = np.hstack((0, np.cumsum(z[:-1]))).astype(int)
-                cn = cnames[cz]
-                FF = zeros(r, r);
-                FY = nans(r, 1);
-                wF = nans(1, r);
-                ivo = [m_([])]*r
-                ivo = [m_([])]*r
-                for i in np.arange(r):
-                    ii = cz[i]
-                    FF[i, i] = np.array([[st0['FF'][ii, ii]]])
-                    FY[i] = np.array([st0['FY'][ii]])
-                    wF[0, i] = np.array([st0['wF'][0, ii]])
-                    ivi[i] = m_([ii])
-                    ivo[i] = np.arange(1, z[i])
-                n2 = ones((r,))
-            else:
-                ivo = [m_([])]*r
-                n2 = [None]*r
-                cn = m_([[]], 0, 1)
-                FFi = [None]*r
-                zz = np.hstack((0, np.cumsum(z)))
-                for i in np.arange(r):
-                    ii = ivi[i]
-                    ivo[i] = np.array(list(set(np.arange(zz[i], zz[i+1])) - set(ii)))
-                    n2[i] = ii.shape[0]
-                    cn = np.vstack((cn, cnames[ii]))
-                    FFi[i] = sm_(st0['FF'], ii, ii)
-                    if i == 0:
-                        FY = sm_(st0['FY'], ii)
-                        wF = sm_(st0['wF'], ii)
-                    else:
-                        FY = np.vstack((FY, sm_(st0['FY'], ii)))
-                        wF = np.hstack((wF, sm_(st0['wF'], ii)))
-                FF = blkdiag(FFi)
+        ivo = np.array(list(set(np.arange(z)) - set(ivi.flatten())))
+        n2 = ivi.shape[0]
+        cn = cnames[ivi]
+        FF = sm_(st0['FF'], ivi, ivi)
+        FY = sm_(st0['FY'], ivi)
+        wF = sm_(st0['wF'], ivi)
         pm, P = mdl(FF, FY, s_min)
         model0 = {'ivi': ivi,
                   'ivo': ivo
                  }
-        st_ext = stmdl(FB='FS1', model=model0, rtp=rtp, FY=FY, P=P, wF=wF, n2=n2, n1=[], st0=st0)
-        model = [htmdl('FS1', [], 'FS', n2, ivi, ivo, cnames, [], pm, st_ext['ovr'])]
+        st_ext = stmdl(FB='FS1', model=model0, rtp='PM', FY=FY, P=P, wF=wF, n2=n2, n1=[], st0=st0)
+        model = [htmdl('FS1', [], 'FS', n2, ivi, ivo, cnames, [], pm, st_ext['ovr'], rtp='PM')]
     else:
         raise Exception('Initial model structure is not defined...')
     return model
@@ -182,6 +133,7 @@ def fSelect(model, st0, met, cnames, s_min, SLE, crit_nbm, iterate):
     if model[-1]['FB'] == 'BE':
         if iterate == 0: iterate = -1
         return model, iterate  # las1t step was BE
+    elif model[-1]['FB'] == 'BE_': model[-1]['FB'] = 'BE'
 
     # todo: calc model(end).st.out when BR is succesfull --> calc in bElimin.m
     # if length(model) > 1 && isempty(model(end).st_in),  model_new = model(end);  model = model(1:end - 1); # successful BE step
@@ -216,6 +168,7 @@ def fSelect(model, st0, met, cnames, s_min, SLE, crit_nbm, iterate):
             ivo1 = ivo[ind[0]]
             ivo = np.delete(ivo, ind, 0)
             model = model + [htmdl('FS1', model[-1], 'FS', n2, ivi, ivo, cnames, ivo1, pmi, sti)]
+
         else:
             iterate = 0
     else:
@@ -237,18 +190,10 @@ def htmdl(mode=None, model0=None, FB=None, n2=None, ivi=None, ivo=None, cnames=N
             model['cname_i'] = m_(cnames)[ivi]
             model['cname_o'] = m_(cnames)[ivo]
         elif rtp == 'PV':
-            z = 0
-            model['cname_i'] = [np.nan]*st['r']
-            for i in range(len(ivi)):
-                ivi_i = ivi[i] + z
-                z += len(ivi[i])
-                model['cname_i'][i] = m_(cnames)[ivi_i]
-            z = 0
-            model['cname_o'] = [np.nan]*st['r']
-            for i in range(len(ivo)):
-                ivo_i = ivo[i] + z
-                z += len(ivo[i])
-                model['cname_o'][i] = m_(cnames)[ivo_i]
+            model['cname_i'] = [np.nan] * st['r']
+            model['cname_o'] = [np.nan] * st['r']
+            for i in range(len(ivi)): model['cname_i'][i] = m_(cnames)[ivi[i]]
+            for i in range(len(ivo)): model['cname_o'][i] = m_(cnames)[ivo[i]]
         model['FB'] = FB
         model['n'] = n2
         model['xio'] = m_(cnames)[ind2] # todo: make it an array 1 x r
@@ -355,8 +300,8 @@ def parin(st0, model):
     wF = st0['wF']
     ivi = model['ivi']
     ivo = model['ivo']
-    n1 = len(ivi) - 1
-    n2 = n1 + 1
+    n2 = model['n']     # n2 = len(ivi)
+    n1 = n2 - 1         # n1 = len(ivi) - 1
     return Nw, r, z, FY, FF, wF, ivi, ivo, n1, n2
 
 
@@ -466,8 +411,8 @@ def stmdl(FB=None, model=None, rtp='PM', FY=None, P=None, wF=None, n2=np.nan, n1
         wFPFY = nans((r,))
         ni_1 = 0
         for i in np.arange(r):
-            ni = len(ivi[i])
-            ii += np.arange(ni_1, ni)
+            ni = ni_1 + len(ivi[i])
+            ii = np.arange(ni_1, ni)
             ni_1 = ni
             if isinstance(ii, int):
                 FYi = m_(FY[ii, :], 1, 1)
@@ -489,7 +434,7 @@ def stmdl(FB=None, model=None, rtp='PM', FY=None, P=None, wF=None, n2=np.nan, n1
     Fo = ssr/sse*v2/v1o
     p_Fo = pvalF(Fo, v1o, v2, 'ot')
     mse = sse/(N - n2)
-    ste = np.sqrt(c_(np.diag(P))*r_(mse))
+    ste = np.sqrt(m_(c_(np.diag(P))*r_(mse), tp='float'))
     R2 = ssr/sst
     R2 = 1 - sse/sst
     R2adj = 1 - (1 - R2)*(Nw - 1)/(Nw - n2)
@@ -504,21 +449,23 @@ def stmdl(FB=None, model=None, rtp='PM', FY=None, P=None, wF=None, n2=np.nan, n1
     if FB == 'FS':
         R2prt = R2 - R2_1
         v1p = np.abs(n2 - n1)
-        t2ss = np.array([np.max(np.hstack((0, (ssm - ssm_1).flatten())))])
+        t2ss = ssm - ssm_1
+        t2ss[t2ss < 0] = 0
         Fp = t2ss/sse*v2/v1p
         p_Fp = pvalF(Fp, v1p, v2, 'ot').flatten()
     elif FB == 'BE':
         R2prt = R2_1 - R2
         v1p = np.abs(n2 - n1)
-        t2ss = np.array([np.max(np.hstack((0, (ssm_1 - ssm).flatten())))])
+        t2ss = ssm_1 - ssm
+        t2ss[t2ss < 0] = 0
         Fp = t2ss/sse_1*v2/v1p
         p_Fp = pvalF(Fp, v1p, v2, 'ot').flatten()
     else:
-        R2prt = None
+        R2prt = m_([None]*r)
         v1p = None
-        t2ss = None
-        Fp = None
-        p_Fp = None
+        t2ss = m_([None]*r)
+        Fp = m_([None]*r)
+        p_Fp = m_([None]*r)
 
     st = {};
     st['ovr'] = {}
@@ -568,31 +515,34 @@ def stmdl(FB=None, model=None, rtp='PM', FY=None, P=None, wF=None, n2=np.nan, n1
 
 
 ###################################################################################
-def swlinr(F=None, U=None, Y=None, W=None, na=None, nb=None, nc=None, pm0=None, cnames=None, mtp=None, rtp='PM', met='SWR', SLE=0.05, SLS=0.05, crit_nbm={'p_Fp':'max','AIC':'any','Cp':'any'}, crit_nbm_tp='min', mdl_init='empty', ivi=m_(tp=int), val_prc=0, s_min=1e-12, dsp=False):
-    N, r = Y.shape
-    if F is not None:
-        if rtp == 'PV': raise Exception('In SWLINR(): Input parameter F is set only when the model is in the parameter-vector form (the parameter rtp = \'PV\').')
-        m = None
-    if cnames is None: cnames = ['var_' + str(i) for i in range(m)]
-    if mdl_init == 'empty' and (isinstance(pm0, (np.ndarray, list)) and np.sum(pm0) < r or isinstance(pm0, (int, float)) and pm0 == 0):
-        if pm0 is not None and np.sum(pm0) < r: wn.warn('In SWLINR(): Intercept(s) introduced in the model as the feature selection starts with "empty" initial model (the parameter mdl_init = \'empty\') that is equivalent to "intercept-only model".')
-        pm0 = ones(r, 1)
+def swlinr(F=None, U=None, Y=None, W=None, m=None, r=None, na=None, nb=None, nc=None, pm0=None, cnames=None, mtp=None, met='SWR', SLE=0.05, SLS=0.05, crit_nbm={'p_Fp':'max','AIC':'any','Cp':'any'}, crit_nbm_tp='min', mdl_init='empty', ivi=m_(tp=int), val_prc=0, s_min=1e-12, dsp=False):
+    if F is None and U is None or Y is None: raise Exception('In SWLINR(): One or more data matrices are not specified.')
+    N = Y.shape[0]
+    if r is None: N = Y.shape[1]
+    if m is None:
+        if F is not None:   m = F.shape[1]
+        else:               m = U.shape[1]    
+
+    F, pm0, ivi, cnames, flg_addint = add_intrcpt(F, pm0, ivi, cnames, mdl_init)
+    # cnames = add_names(cnames, r, flg_addint)
+    if cnames is None:
         if F is not None:
-            F = np.hstack((np.ones((N, 1)), F))
-            cnames = ['intercept'] + cnames
-            ivi = np.hstack((1, ivi + 1))
+            cnames = ['var_' + str(i) for i in range(F.shape[1] - flg_addint)]
+            if flg_addint: cnames = ['intercept'] + cnames
+        else:
+            cnames = names(na=na, nb=nb, pm0=pm0, r=r, m=m, rtp='PM')
+    else:
+        pass
 
-    if U is not None and mtp=='arx':
+    # factor & weight matrices
+    if U is not None and mtp == 'arx':
         n = np.max(np.hstack((na, nb)))
-        if rtp == 'PM':     F = dmpm(U, Y, na=na, nb=nb, pm0=pm0)
-        elif rtp == 'PV':   F = dmpv(U, Y, na=na, nb=nb, pm0=pm0)
-        m = U.shape[1]
+        F = dmpm(U, Y, na=na, nb=nb, pm0=pm0)
         Y = Y[n:, :]
-        if F is not None: W = W[n:, :]
-
-    st0 = stats(F, Y, W, rtp=rtp, m=m, r=r, na=na, nb=nb, nc=nc, pm0=pm0, s_min=s_min)
-
-    model = firstmdl(st0, mdl_init, s_min, cnames, ivi, rtp)
+        W = W[n:, :]
+            
+    st0 = stats(F, Y, W, rtp='PM', m=m, r=r, na=na, nb=nb, nc=nc, pm0=pm0, ivi=ivi, s_min=s_min)
+    model = firstmdl(st0, mdl_init, s_min, cnames, ivi)
     iterate = 1
     while iterate > 0:
         model, iterate = bElimin(model, st0, met, cnames, s_min, SLS, crit_nbm, iterate)
@@ -615,9 +565,9 @@ def visualz(model, iterate, dsp, mdl_init):
         print('Step: ', step, '  =================================================================================================================')
         if model['FB'] == 'FS' and step != 0:
             print('Added factor: ', model['xio'])
-        elif model['FB'] == 'BE' and step != 0:
+        elif (model['FB'] == 'BE' or model['FB'] == 'BE_stop') and step != 0:
             print('Removed factor: ', model['xio'])
-        elif mdl_init == 'init':
+        elif mdl_init == 'init' and step == 0:
             print('Initial model: ', model['cname_i'].flatten())
         elif model['FB'] == 'FS' and step == 0:
             print('Initial model: intercept')
@@ -659,23 +609,30 @@ def visualz(model, iterate, dsp, mdl_init):
             print(' ')
         if 'st' in model and 'in' in model['st'] and dsp == 'all':
             print('--- Partial model measures ---')
-            Variable = c_(model['cname_i'])
-            Estimates = model['pm']
+            r = model['r']
+            Variable = repmat(c_(model['cname_i']), r, 1)
+            Estimates = vec(model['pm'])
             st = model['st']['in']
             if st is not None:
                 n = len(st)
-                T2SS = nans((n, 1))
-                PartialF = nans((n, 1))
-                pval_PartialF = nans((n, 1))
-                PartialR2 = nans((n, 1))
-                for i in np.arange(n):
-                    T2SS[i] = st[i]['t2ss']
-                    PartialF[i] = st[i]['Fp']
-                    pval_PartialF[i] = st[i]['p_Fp']
-                    PartialR2[i] = st[i]['R2prt']
-                SandardError = model['st']['ovr']['STE']
+                T2SS = nans(n*r, 1)
+                PartialF = nans(n*r, 1)
+                pval_PartialF = nans(n*r, 1)
+                PartialR2 = nans(n*r, 1)
+                Variables = [None]*n*r
+                h = 0
+                for j in np.arange(r):
+                    for i in np.arange(n):
+                        T2SS[h] = st[i]['t2ss'][j]
+                        PartialF[h] = st[i]['Fp'][j]
+                        pval_PartialF[h] = st[i]['p_Fp'][j]
+                        PartialR2[h] = st[i]['R2prt'][j]
+                        Variables[h] = Variable[i,0] + ' -> y' + str(j+1) + '(k)'
+                        h += 1
+                Variables = c_(Variables)
+                SandardError = vec(model['st']['ovr']['STE'])
                 if not len(PartialF) == 0 and PartialF.shape == Variable.shape:
-                    print(pd.DataFrame({'Variable': Variable[:, 0],
+                    print(pd.DataFrame({'Variable': Variables[:, 0],
                                         'Estimates': Estimates[:, 0],
                                         'SandardError': SandardError[:, 0],
                                         'T2SS': T2SS[:, 0],
@@ -694,15 +651,21 @@ def visualz(model, iterate, dsp, mdl_init):
             Variable = c_(model['cname_o'])
             st = model['st']['out']
             n = len(st)
-            T2SS = nans((n, 1))
-            PartialF = nans((n, 1))
-            pval_PartialF = nans((n, 1))
-            PartialR2 = nans((n, 1))
-            for i in np.arange(n):
-                T2SS[i] = st[i]['t2ss']
-                PartialF[i] = st[i]['Fp']
-                pval_PartialF[i] = st[i]['p_Fp']
-                PartialR2[i] = st[i]['R2prt']
+            T2SS = nans(n * r, 1)
+            PartialF = nans(n * r, 1)
+            pval_PartialF = nans(n * r, 1)
+            PartialR2 = nans(n * r, 1)
+            Variables = [None] * n * r
+            h = 0
+            for j in np.arange(r):
+                for i in np.arange(n):
+                    T2SS[h] = st[i]['t2ss'][j]
+                    PartialF[h] = st[i]['Fp'][j]
+                    pval_PartialF[h] = st[i]['p_Fp'][j]
+                    PartialR2[h] = st[i]['R2prt'][j]
+                    Variables[h] = Variable[i, 0] + ' -> y' + str(j + 1) + '(k)'
+                    h += 1
+            Variables = c_(Variables)
             if not len(PartialF) == 0 and len(PartialF) == len(Variable):
                 print(pd.DataFrame({'Variable': Variable[:, 0],
                                     'T2SS': T2SS[:, 0],
