@@ -72,9 +72,9 @@ def bElimin(model, step, x, y, w, met, SLS, pm0, st0, cnames, metpminit, met_opt
 
 
 ###################################################################################
-def firstmdl(x, y, w, cnames, st0, smin, mtp, ivi, pm0, metpminit, met_opt, maxiter, fcnv, afcnv, xcnv, gcnv, sc,
+def firstmdl(x, y, w, cnames, st0, smin, mdl_init, ivi, pm0, metpminit, met_opt, maxiter, fcnv, afcnv, xcnv, gcnv, sc,
              dsp_op, smeth, su, p_decr, p_incr):
-    if mtp == 'full':
+    if mdl_init == 'full':
         z = st0['z']
         ivi = np.arange(z)
         ivo = np.array([]).astype(int)
@@ -85,7 +85,7 @@ def firstmdl(x, y, w, cnames, st0, smin, mtp, ivi, pm0, metpminit, met_opt, maxi
         st_ext = stmdl('BE1', x, y, ym, w, st0, model, n2, ivi, ivo, smin)
         model = [htmdl('BE1', [], 'BE', n2, ivi, ivo, cnames, [], model['pm'], st_ext)]
     else:
-        if mtp == 'empty':
+        if mdl_init == 'empty':
             z = st0['z']
             ivi = np.array([0])
             ivo = np.arange(1, z)
@@ -97,7 +97,7 @@ def firstmdl(x, y, w, cnames, st0, smin, mtp, ivi, pm0, metpminit, met_opt, maxi
             pm = st0['modelInt']['pm']
             model = [htmdl('FS1', [], 'FS', n2, ivi, ivo, cnames, [], pm, st_ext)]
         else:
-            if mtp == 'init':
+            if mdl_init == 'init':
                 z = st0['z']
                 ivo = np.setxor1d(c_(np.arange(z)), ivi)
                 n2 = ivi.shape[0]
@@ -315,7 +315,7 @@ def scrChiSq(x, y, ym0, ym, w, st0, model, mode, ivi, ivo, smin):
 
 
 ###################################################################################
-def swlogr(x, y, w, cnames=None, SLE=0.05, SLS=0.05, pm0=1, dsp=False, nbm_crit=np.array([]), mtp='full', met='SWR',
+def swlogr(x, y, w, cnames=None, SLE=0.05, SLS=0.05, pm0=1, dsp=False, nbm_crit=np.array([]), mdl_init='empty', met='SWR',
            metpminit='zero', ivi=np.empty([1, ]).astype(int), met_opt='newt', gcnv=1e-8, fcnv=0, afcnv=0, xcnv=0,
            maxiter=100, smin=1e-12, dsp_op=np.array([1, 2]), smeth='fprev2', p_incr=1.2, p_decr=0.6, sl=1e-12, su=1,
            sc=None):
@@ -327,7 +327,7 @@ def swlogr(x, y, w, cnames=None, SLE=0.05, SLS=0.05, pm0=1, dsp=False, nbm_crit=
         ivi = np.hstack((1, ivi + 1))
     iterate = 1
     st0 = stats(x, y, w, smin, pm0, cnames, metpminit, ivi, met_opt, maxiter, fcnv, afcnv, xcnv, gcnv, sc, dsp_op, dsp, smeth, su, p_decr, p_incr)
-    model, step, st0 = firstmdl(x, y, w, cnames, st0, smin, mtp, ivi, pm0, metpminit, met_opt, maxiter, fcnv, afcnv, xcnv, gcnv, sc, dsp_op, smeth, su, p_decr, p_incr)
+    model, step, st0 = firstmdl(x, y, w, cnames, st0, smin, mdl_init, ivi, pm0, metpminit, met_opt, maxiter, fcnv, afcnv, xcnv, gcnv, sc, dsp_op, smeth, su, p_decr, p_incr)
     while iterate:
         model, step, st0, iterate = bElimin(model, step, x, y, w, met, SLS, pm0, st0, cnames, metpminit, met_opt,
                                             maxiter, fcnv, afcnv, xcnv, gcnv, dsp, iterate, sc, dsp_op, smeth, su,
