@@ -40,15 +40,13 @@ def dspl(flg, **kwargs):
             elif 'B' == msg:
                 print('Stopping Rule (MaxIter = ', kwargs['maxiter'], ') satisfied. /', itr, '/')
             elif 'C' == msg:
-                print('Convergence criterion (FCONV = ', kwargs['fcnv'], ') satisfied. /',
-                      np.abs(F - FF[0])/np.max(np.abs(FF[0]), initial=1e-6), '/')
+                print('Convergence criterion (FCONV = ', kwargs['fcnv'], ') satisfied. /', np.abs(F - FF[0])/np.max(np.abs(FF[0]), initial=1e-6), '/')
             elif 'D' == msg:
                 print('Convergence criterion (ABSFCONV = ', kwargs['afcnv'], ') satisfied. /', np.abs(F - FF[0]), '/')
             elif 'E' == msg:
                 print('Convergence criterion (XCONV = ', kwargs['xcnv'], ') satisfied. /', (x - xx[:, 0]).T/x, '/')
             elif 'F' == msg:
-                print('Convergence criterion (GCONV = ', kwargs['gcnv'], ') satisfied. /',
-                      np.abs(g.T@Hinv@g)/(np.abs(F) + 1e-6), '/')
+                print('Convergence criterion (GCONV = ', kwargs['gcnv'], ') satisfied. /', np.abs(g.T@Hinv@g)/(np.abs(F) + 1e-6), '/')
             elif 'G' == msg:
                 print('Termination: reaching the uncretainty level...')
             elif 'H' == msg:
